@@ -189,9 +189,13 @@ function test_compress_decompress_fixed() {
 
 function build_code ()
 {
-    echo "--------------------------------------------------------------------"
-    echo "Build code ..."
-    make || exit 1
+	# Do not build the code, there might be some options e.g. not
+	# using libcxl, which must be set such that the build can
+	# Succeed on some special test-systems we have.
+
+    #echo "--------------------------------------------------------------------"
+    #echo "Build code ..."
+    #make || exit 1
 
     echo "--------------------------------------------------------------------"
     if [ -f test_data.bin ]; then
@@ -397,7 +401,7 @@ function zlib_append ()
     local params=$2
 
     # Use default settings ...
-    # Set size large enough that hardware inflate is realy used
+    # Set size large enough that hardware inflate is really used
     #
     # hhh [0x3ffff1c655d8] loops=0 flush=1 Z_PARTIAL_FLUSH
     # hhh [0x3ffff1c655d8] *** giving out 100 bytes ...
